package tech.nevets.jconf;

import com.google.gson.*;

import java.io.*;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;

public class JsonConfig implements Config {
    private static final Gson GSON = new GsonBuilder().setPrettyPrinting().create();

    private JsonObject confObj;
    private final String path;

    public JsonConfig(String path, String defaultData) {
        this.path = path;
        File f = new File(path);
        if (!f.exists()) {
            try {
                f.getParentFile().mkdirs();
                boolean newFileCreated = f.createNewFile();
                if (newFileCreated && defaultData != null) {
                    FileWriter fw = new FileWriter(f);
                    fw.write(defaultData);
                    fw.flush();
                    fw.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        try {
            confObj = GSON.fromJson(new FileReader(f), JsonObject.class);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }

    public JsonConfig(String name) {
        this(name, null);
    }

    public String getString(String key) {
        return confObj.get(key).getAsString();
    }

    public void setString(String key, String value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public String[] getStringArray(String key) {
        ArrayList<String> al = new ArrayList<>();
        for (JsonElement je : getJsonArray(key).asList()) {
            al.add(je.getAsString());
        }
        return al.toArray(new String[0]);
    }

    public int getInt(String key) {
        return confObj.get(key).getAsInt();
    }

    public void setInt(String key, int value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public boolean getBoolean(String key) {
        return confObj.get(key).getAsBoolean();
    }

    public void setBoolean(String key, boolean value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public Number getNumber(String key) {
        return confObj.get(key).getAsNumber();
    }

    public void setNumber(String key, Number value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public byte getByte(String key) {
        return confObj.get(key).getAsByte();
    }

    public void setByte(String key, byte value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public short getShort(String key) {
        return confObj.get(key).getAsShort();
    }

    public void setShort(String key, short value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public long getLong(String key) {
        return confObj.get(key).getAsLong();
    }

    public void setLong(String key, long value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public float getFloat(String key) {
        return confObj.get(key).getAsFloat();
    }

    public void setFloat(String key, float value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public double getDouble(String key) {
        return confObj.get(key).getAsDouble();
    }

    public void setDouble(String key, double value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public BigInteger getBigInt(String key) {
        return confObj.get(key).getAsBigInteger();
    }

    public void setBigInt(String key, BigInteger value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public BigDecimal getBigDecimal(String key) {
        return confObj.get(key).getAsBigDecimal();
    }

    public void setBigDecimal(String key, BigDecimal value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public char getChar(String key) {
        return confObj.get(key).getAsCharacter();
    }

    public void setChar(String key, char value) {
        confObj.addProperty(key, value);
        saveConfig();
    }

    public JsonElement getJsonElement(String key) {
        return confObj.get(key);
    }

    public void setJsonElement(String key, JsonElement value) {
        confObj.add(key, value);
        saveConfig();
    }

    public JsonObject getJsonObject(String key) {
        return confObj.get(key).getAsJsonObject();
    }

    public JsonArray getJsonArray(String key) {
        return confObj.get(key).getAsJsonArray();
    }

    public void saveConfig() {
        try {
            File f = new File(path);
            FileWriter fw = new FileWriter(f);
            fw.write(GSON.toJson(confObj));
            fw.flush();
            fw.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void reloadConfig() {
        try {
            confObj = GSON.fromJson(new FileReader(path), JsonObject.class);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }
}
